﻿using System;
using System.Data;
using System.Data.SqlClient;
using System.Data.SqlTypes;
using Microsoft.SqlServer.Server;

/*============================================================================
  File:     CovarFloat.cs

  Summary:  Implementation of a floating point version of a multi-parameter 
			user-defined aggregate, covar_pop. UDaggs and UDTs that use
            only floating point fields do not need an IBinarySerialize
            implementation.
			
			NOTE: Deploying multi-parameter user-defined aggregates is not 
			supported by Visual Studio autodeploy. Use the script provided and manual
			assembly and function deployment.

  Date:     August 2008

  SQL Server Version: 10.0.1600.22 (RTM)
------------------------------------------------------------------------------
  Written by Bob Beauchemin, SQLskills.com

  For more scripts and sample code, check out http://www.SQLskills.com

  This script is intended only as a supplement to SQL Server Jumpstart program. 
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/


[Serializable]
[Microsoft.SqlServer.Server.SqlUserDefinedAggregate(Format.Native)]
public struct CovarFloat
{
    double theanswer;
    double sumproduct;
    double sum1;
    double sum2;
    int count;

    public void Init()
    {
        // Put your code here
    }

    public void Accumulate(Nullable<double> Value1, Nullable<double> Value2)
    {
        if (!Value1.HasValue)
            return;
        if (!Value2.HasValue)
            return;
        sum1 += Value1.Value;
        sum2 += Value2.Value;
        sumproduct += (Value1.Value * Value2.Value);
        count++;
    }

    public void Merge(CovarFloat Group)
    {
        sum1 += Group.sum1;
        sum2 += Group.sum2;
        sumproduct += Group.sumproduct;
        count += Group.count;
    }

    public Nullable<double> Terminate()
    {
        if (count == 0)
            return 0;

        theanswer = (sumproduct - sum2 * sum1 / count) / count;
        return theanswer;
    }

}
